/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.sensor;

import java.io.IOException;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.util.NbBundle;

/**
 * A base DataObject for SensorData When constructed a background task is
 * started to process the content into a displayable image.
 *
 * @author Timon Veenstra
 */
public abstract class SensorDataObject extends MultiDataObject {

    private SensorPalette palette;
    private SensorGrid grid;
    public static final String PROP_PALETTE_CHANGED = "paletteChanged";
    private static final String PALETTE_PROPERTY = "property";
    private static final String WARING_SAVE_PALETTE_MESSAGE = NbBundle.getMessage(SensorDataObject.class, "sensor.palette.warning.save.palette");
    private static final AppLogger LOG = AppLogFactory.getLogger(SensorDataObject.class);

    public SensorDataObject(FileObject fo, MultiFileLoader loader) throws DataObjectExistsException {
        super(fo, loader);
    }

    /**
     * Get the sensor grid for a the {@link SensorProcessor} returned by {@link SensorDataObject#getProcessor()}
     *
     * @return
     */
    public SensorGrid getSensorGrid() {
        if (grid == null) {
            grid = new SensorGrid(getProcessor());
        }
        return grid;
    }

    /**
     * Get the {@link SensorProcessor} from the implementing class Used in {@link SensorDataObject#getSensorGrid()}
     * to create the appropriate {@link SensorGrid}
     *
     * @return
     */
    protected abstract SensorProcessor getProcessor();

    /**
     * Get the palette for the SensorDataObject
     *
     * @return
     */
    public SensorPalette getPalette() {
        if (palette == null) {
            palette = (SensorPalette) getPrimaryFile().getAttribute(PALETTE_PROPERTY);
            if (palette == null) {
                palette = SensorPalette.PALLET00_10;
            }
        }
        return palette;
    }

    /**
     * Set the palette for the SensorDataobject
     *
     * @param palette
     */
    public void setPalette(SensorPalette palette) {
        SensorPalette old = this.palette;
        this.palette = palette;
        this.firePropertyChange(PROP_PALETTE_CHANGED, old, palette);

        try {
            getPrimaryFile().setAttribute(PALETTE_PROPERTY, palette);
        } catch (IOException ex) {
            LOG.warning(WARING_SAVE_PALETTE_MESSAGE);
        }
    }
}
